@@ -34,7 +34,7 @@ def run():
       grafana-wtf [options] explore datasources
       grafana-wtf [options] explore dashboards
       grafana-wtf [options] find [<search-expression>]
-      grafana-wtf [options] replace <search-expression> <replacement>
+      grafana-wtf [options] replace <search-expression> <replacement> [--dry-run]
       grafana-wtf [options] log [<dashboard_uid>] [--number=<count>]
       grafana-wtf --version
       grafana-wtf (-h | --help)
@@ -45,9 +45,10 @@ def run():
       --select-dashboard=<uuid>         Restrict operation to dashboard by UID.
                                         Can be a list of comma-separated dashboard UIDs.
       --format=<format>                 Output format. [default: json]
-      --cache-ttl=<cache-ttl>           Time-to-live for the request cache in seconds. [default: 300]
+      --cache-ttl=<cache-ttl>           Time-to-live for the request cache in seconds. [default: 60]
       --drop-cache                      Drop cache before requesting resources
       --concurrency=<concurrency>       Run multiple requests in parallel. [default: 5]
+      --dry-run                         Dry-run mode for the `replace` subcommand.
       --verbose                         Enable verbose mode
       --version                         Show version information
       --debug                           Enable debug messages
@@ -121,6 +122,9 @@ def run():
       # Replace string within specific dashboard.
       grafana-wtf --select-dashboard=_JJ22OZZk replace grafana-worldmap-panel grafana-map-panel
 
+      # Preview the changes beforehand, using the `--dry-run` option.
+      grafana-wtf --select-dashboard=_JJ22OZZk replace grafana-worldmap-panel grafana-map-panel --dry-run
+
     Display edit history:
 
       # Display 50 most recent changes across all dashboards.
@@ -181,7 +185,6 @@ def run():
     output_format = options["format"]
 
     if options.find or options.replace:
-
         if options.select_dashboard:
             # Restrict scan to list of dashboards.
             dashboard_uids = read_list(options.select_dashboard)
@@ -203,7 +206,7 @@ def run():
         report.display(options.search_expression, result)
 
     if options.replace:
-        engine.replace(options.search_expression, options.replacement)
+        engine.replace(options.search_expression, options.replacement, dry_run=options.dry_run)
         engine.clear_cache()
 
     if options.log:
