@@ -5,9 +5,10 @@ import asyncio
 import dataclasses
 import json
 import logging
+import warnings
 from collections import OrderedDict
 from concurrent.futures.thread import ThreadPoolExecutor
-from urllib.parse import urljoin, urlparse
+from urllib.parse import parse_qs, urljoin, urlparse
 
 import colored
 import requests
@@ -16,6 +17,8 @@ from grafana_client.api import GrafanaApi
 from grafana_client.client import GrafanaClientError, GrafanaUnauthorizedError
 from munch import Munch, munchify
 from tqdm import tqdm
+from tqdm.contrib.logging import tqdm_logging_redirect
+from urllib3.exceptions import InsecureRequestWarning
 
 from grafana_wtf.model import (
     DashboardDetails,
@@ -24,7 +27,7 @@ from grafana_wtf.model import (
     DatasourceItem,
     GrafanaDataModel,
 )
-from grafana_wtf.util import JsonPathFinder
+from grafana_wtf.util import JsonPathFinder, as_bool
 
 log = logging.getLogger(__name__)
 
@@ -45,13 +48,16 @@ class GrafanaEngine:
         self.debug = log.getEffectiveLevel() == logging.DEBUG
         self.progressbar = not self.debug
 
-    def enable_cache(self, expire_after=300, drop_cache=False):
+    def enable_cache(self, expire_after=60, drop_cache=False):
         if expire_after is None:
-            log.info(f"Setting up response cache to never expire (infinite caching)")
+            log.info(f"Configure response cache to never expire (infinite caching)")
         else:
-            log.info(f"Setting up response cache to expire after {expire_after} seconds")
-        requests_cache.install_cache(expire_after=expire_after)
+            log.info(f"Configure response cache to expire after {expire_after} seconds")
+        requests_cache.install_cache(expire_after=expire_after, use_cache_dir=True)
+        cache_database_file = requests_cache.get_cache().db_path
+        log.info(f"Response cache database location is {cache_database_file}")
         if drop_cache:
+            log.info("Dropping response cache")
             self.clear_cache()
 
         return self
@@ -77,14 +83,21 @@ class GrafanaEngine:
             password = url.password or "admin"
             auth = (username, password)
 
+        verify = as_bool(parse_qs(url.query).get("verify", [True])[0])
+        if verify is False:
+            warnings.filterwarnings("ignore", category=InsecureRequestWarning)
         grafana = GrafanaApi(
-            auth, protocol=url.scheme, host=url.hostname, port=url.port, url_path_prefix=url.path.lstrip("/")
+            auth,
+            protocol=url.scheme,
+            host=url.hostname,
+            port=url.port,
+            url_path_prefix=url.path.lstrip("/"),
+            verify=verify,
         )
 
         return grafana
 
     def setup(self):
-
         self.grafana = self.grafana_client_factory(self.grafana_url, grafana_token=self.grafana_token)
 
         # Configure a larger HTTP request pool.
@@ -99,7 +112,8 @@ class GrafanaEngine:
 
     def start_progressbar(self, total):
         if self.progressbar:
-            self.taqadum = tqdm(total=total)
+            with tqdm_logging_redirect():
+                self.taqadum = tqdm(total=total)
 
     def scan_common(self):
         self.scan_dashboards()
@@ -157,7 +171,6 @@ class GrafanaEngine:
                 )
 
     def scan_dashboards(self, dashboard_uids=None):
-
         log.info("Scanning dashboards")
         try:
             if dashboard_uids is not None:
@@ -225,7 +238,6 @@ class GrafanaEngine:
         # https://hackernoon.com/how-to-run-asynchronous-web-requests-in-parallel-with-python-3-5-without-aiohttp-264dc0f8546
         log.info(f"Fetching dashboards in parallel with {self.concurrency} concurrent requests")
         with ThreadPoolExecutor(max_workers=self.concurrency) as executor:
-
             # Set any session parameters here before calling `fetch`
             loop = asyncio.get_event_loop()
             # START_TIME = default_timer()
@@ -242,7 +254,6 @@ class GrafanaEngine:
 
 class GrafanaWtf(GrafanaEngine):
     def info(self):
-
         try:
             health = self.grafana.client.GET("/health")
         except Exception as ex:
@@ -296,7 +307,6 @@ class GrafanaWtf(GrafanaEngine):
         return response
 
     def version(self):
-
         try:
             health = self.grafana.client.GET("/health")
         except Exception as ex:
@@ -325,7 +335,9 @@ class GrafanaWtf(GrafanaEngine):
 
         return results
 
-    def replace(self, expression, replacement):
+    def replace(self, expression, replacement, dry_run: bool = False):
+        if dry_run:
+            log.info("Dry-run mode enabled, skipping any actions")
         log.info(f'Replacing "{expression}" by "{replacement}" within Grafana at "{self.grafana_url}"')
         for dashboard in self.data.dashboards:
             payload_before = json.dumps(dashboard)
@@ -335,7 +347,8 @@ class GrafanaWtf(GrafanaEngine):
                 continue
             dashboard_new = json.loads(payload_after)
             dashboard_new["message"] = f'grafana-wtf: Replaced "{expression}" by "{replacement}"'
-            self.grafana.dashboard.update_dashboard(dashboard=dashboard_new)
+            if not dry_run:
+                self.grafana.dashboard.update_dashboard(dashboard=dashboard_new)
 
     def log(self, dashboard_uid=None):
         if dashboard_uid:
@@ -388,7 +401,6 @@ class GrafanaWtf(GrafanaEngine):
         return r
 
     def explore_datasources(self):
-
         # Prepare indexes, mapping dashboards by uid, datasources by name
         # as well as dashboards to datasources and vice versa.
         ix = Indexer(engine=self)
@@ -422,7 +434,6 @@ class GrafanaWtf(GrafanaEngine):
         return response
 
     def explore_dashboards(self):
-
         # Prepare indexes, mapping dashboards by uid, datasources by name
         # as well as dashboards to datasources and vice versa.
         ix = Indexer(engine=self)
@@ -430,7 +441,6 @@ class GrafanaWtf(GrafanaEngine):
         # Compute list of exploration items, looking for dashboards with missing data sources.
         results = []
         for uid in sorted(ix.dashboard_by_uid):
-
             dashboard = ix.dashboard_by_uid[uid]
             datasource_items = ix.dashboard_datasource_index[uid]
 
@@ -482,28 +492,42 @@ class Indexer:
         self.index()
 
     def index(self):
-        self.index_dashboards()
         self.index_datasources()
+        self.index_dashboards()
+        self.index_crossref()
 
-    @staticmethod
-    def collect_datasource_items(element):
+    def collect_datasource_items(self, element):
+        element = element or []
         items = []
         for node in element:
+            ds = None
+
+            # Directly defined datasources.
             if "datasource" in node and node["datasource"]:
                 ds = node.datasource
                 if isinstance(ds, Munch):
                     ds = dict(ds)
-                if ds not in items:
-                    items.append(ds)
+
+            # Datasources defined as variables.
+            if "type" in node and node["type"] == "datasource":
+                ds_name = node.get("current", {}).get("value")
+                datasource = self.datasource_by_name.get(ds_name, {})
+                ds = dict(
+                    type=datasource.get("type"),
+                    uid=datasource.get("uid"),
+                    name=datasource.get("name"),
+                    url=datasource.get("url"),
+                )
+
+            if ds is not None and ds not in items:
+                items.append(ds)
         return items
 
     def index_dashboards(self):
-
         self.dashboard_by_uid = {}
         self.dashboard_datasource_index = {}
 
         for dbdetails in self.engine.dashboard_details():
-
             dashboard = dbdetails.dashboard
 
             if dashboard.meta.isFolder:
@@ -527,7 +551,6 @@ class Indexer:
             self.dashboard_datasource_index[uid] = results
 
     def index_datasources(self):
-
         self.datasource_by_ident = {}
         self.datasource_by_uid = {}
         self.datasource_by_name = {}
@@ -540,6 +563,7 @@ class Indexer:
                 self.datasource_by_ident[datasource.uid] = datasource
                 self.datasource_by_uid[datasource.uid] = datasource
 
+    def index_crossref(self):
         for dashboard_uid, datasource_items in self.dashboard_datasource_index.items():
             datasource_item: DatasourceItem
             for datasource_item in datasource_items:
